@@ -33,13 +33,15 @@ VARIABLE_ROUTE = '/{variable_id}'
 MODE_ROUTE = '/{mode_code}'
 RUN_ROUTE = '/r/{run_slug}'
 STYLE_ROUTE = '/s/{style_name}.css'
-MUTATIONS_ROUTE = '/mutations'
+MUTATION_ROUTE = '/mutations{uri}.json'
 
 
 MODE_NAMES = 'input', 'output', 'log', 'debug'
 MODE_NAME_BY_CODE = {_[0]: _ for _ in MODE_NAMES}
 MODE_CODE_BY_NAME = {k: v for v, k in MODE_NAME_BY_CODE.items()}
-PING_INTERVAL_IN_SECONDS = 1
+MINIMUM_PING_INTERVAL_IN_SECONDS = 1
+MAXIMUM_PING_INTERVAL_IN_SECONDS = 30
+MAXIMUM_MUTATION_AGE_IN_SECONDS = 60
 
 
 FUNCTION_BY_NAME = {
