@@ -1,14 +1,13 @@
 # TODO: Show runs with command line option
-# TODO: Let user customize root template
 # TODO: Add unit tests
 import json
 from functools import partial
 from invisibleroads_macros_disk import make_random_folder
 from itertools import count
 from logging import getLogger
-from os import getcwd
 from pyramid.httpexceptions import HTTPBadRequest, HTTPNotFound
 from pyramid.response import FileResponse, Response
+from time import time
 
 from ..constants import (
     AUTOMATION_ROUTE,
@@ -16,6 +15,7 @@ from ..constants import (
     ID_LENGTH,
     MODE_NAME_BY_CODE,
     MODE_ROUTE,
+    MUTATION_ROUTE,
     RUN_ROUTE,
     STYLE_ROUTE,
     VARIABLE_ID_PATTERN,
@@ -24,7 +24,7 @@ from ..exceptions import CrossComputeDataError
 from ..macros.iterable import extend_uniquely, find_item
 from ..macros.web import get_html_from_markdown
 from ..routines.batch import DiskBatch
-from ..routines.configuration import BatchDefinition
+from ..routines.configuration import BatchDefinition, VariableDefinition
 from ..routines.variable import (
     Element,
     VariableView,
@@ -34,12 +34,10 @@ from ..routines.variable import (
 class AutomationRoutes():
 
     def __init__(
-            self, configuration, automation_definitions,
-            automation_queue, timestamp_object):
+            self, configuration, automation_definitions, automation_queue):
         self.configuration = configuration
         self.automation_definitions = automation_definitions
         self.automation_queue = automation_queue
-        self._timestamp_object = timestamp_object
 
     def includeme(self, config):
         config.include(self.configure_root)
@@ -49,20 +47,14 @@ class AutomationRoutes():
         config.include(self.configure_runs)
 
     def configure_root(self, config):
-        template_path_by_id = self.configuration.template_path_by_id
-        if 'root' in template_path_by_id:
-            config.add_jinja2_search_path(getcwd())
-            template_path = str(template_path_by_id['root'])
-        else:
-            template_path = 'crosscompute:templates/root.jinja2'
-
+        configuration = self.configuration
         config.add_route('root', '/')
 
         config.add_view(
             self.see_root,
             request_method='GET',
             route_name='root',
-            renderer=template_path)
+            renderer=configuration.get_template_path('root'))
 
     def configure_styles(self, config):
         config.add_route(
@@ -147,7 +139,8 @@ class AutomationRoutes():
             'title_text': self.configuration.get('name', 'Automations'),
             'automations': self.automation_definitions,
             'css_uris': css_uris,
-            'timestamp_value': self._timestamp_object.value,
+            'mutation_uri': MUTATION_ROUTE.format(uri=''),
+            'mutation_timestamp': time(),
         }
 
     def see_style(self, request):
@@ -196,14 +189,16 @@ class AutomationRoutes():
     def see_automation(self, request):
         automation_definition = self.get_automation_definition_from(request)
         css_uris = automation_definition.css_uris
+        uri = automation_definition.uri
         return {
             'name': automation_definition.name,
-            'uri': automation_definition.uri,
+            'uri': uri,
             'batches': automation_definition.batch_definitions,
             'runs': automation_definition.run_definitions,
             'title_text': automation_definition.name,
             'css_uris': css_uris,
-            'timestamp_value': self._timestamp_object.value,
+            'mutation_uri': MUTATION_ROUTE.format(uri=uri),
+            'mutation_timestamp': time(),
         }
 
     def see_automation_batch_mode(self, request):
@@ -213,14 +208,16 @@ class AutomationRoutes():
         batch = DiskBatch(automation_definition, batch_definition)
         base_uri = request.registry.settings['base_uri']
         mode_name = self.get_mode_name_from(request)
+        uri = request.path
         for_print = 'p' in request.params
         return {
             'title_text': batch_definition.name,
             'automation_definition': automation_definition,
             'batch_definition': batch_definition,
-            'uri': request.path,
+            'uri': uri,
             'mode_name': mode_name,
-            'timestamp_value': self._timestamp_object.value,
+            'mutation_uri': MUTATION_ROUTE.format(uri=uri),
+            'mutation_timestamp': time(),
         } | render_mode_dictionary(batch, base_uri, mode_name, for_print)
 
     def see_automation_batch_mode_variable(self, request):
@@ -232,18 +229,25 @@ class AutomationRoutes():
             mode_name)
         matchdict = request.matchdict
         variable_id = matchdict['variable_id']
-        try:
-            variable_definition = find_item(
-                variable_definitions, 'id', variable_id,
-                normalize=str.casefold)
-        except StopIteration:
-            raise HTTPNotFound
+        if mode_name == 'debug' and variable_id == 'return_code':
+            variable_definition = VariableDefinition({
+                'id': 'return_code',
+                'view': 'number',
+                'path': 'variables.dictionary',
+            }, mode_name='debug')
+        else:
+            try:
+                variable_definition = find_item(
+                    variable_definitions, 'id', variable_id,
+                    normalize=str.casefold)
+            except StopIteration:
+                raise HTTPNotFound
         batch = DiskBatch(automation_definition, batch_definition)
         variable_data = batch.get_data(variable_definition)
         if 'path' in variable_data:
             return FileResponse(variable_data['path'], request=request)
         if 'value' in variable_data:
-            return Response(variable_data['value'])
+            return Response(str(variable_data['value']))
         if 'error' in variable_data:
             raise HTTPNotFound
         raise HTTPBadRequest
