@@ -6,7 +6,7 @@ from invisibleroads_macros_log import format_path
 from logging import getLogger
 from nbconvert import PythonExporter
 from nbformat import read as load_notebook, NO_CONVERT
-from os import environ
+from os import environ, symlink
 from os.path import relpath, splitext
 from pathlib import Path
 from ruamel.yaml import YAML
@@ -71,6 +71,7 @@ class AutomationDefinition(Definition):
             validate_variable_views,
             validate_templates,
             validate_batches,
+            validate_datasets,
             validate_scripts,
             validate_environment_variables,
             validate_display_styles,
@@ -89,6 +90,14 @@ class AutomationDefinition(Definition):
                     MODE_NAME))
         return variable_definitions
 
+    def get_template_path(self, template_id):
+        template_path_by_id = self.template_path_by_id
+        if template_id in template_path_by_id:
+            template_path = str(self.folder / template_path_by_id[template_id])
+        else:
+            template_path = f'crosscompute:templates/{template_id}.jinja2'
+        return template_path
+
     def get_template_text(self, mode_name):
         automation_folder = self.folder
         variable_definitions = self.get_variable_definitions(
@@ -98,6 +107,20 @@ class AutomationDefinition(Definition):
         return get_template_text(
             template_definitions, automation_folder, variable_definitions)
 
+    def update_datasets(self):
+        automation_folder = self.folder
+        for dataset_definition in self.dataset_definitions:
+            if 'path' in dataset_definition.reference:
+                reference_path = dataset_definition.reference['path']
+                source_path = automation_folder / reference_path
+                target_path = automation_folder / dataset_definition.path
+                if target_path.is_symlink():
+                    target_path.unlink()
+                elif target_path.exists():
+                    continue
+                target_folder = target_path.parent
+                symlink(source_path.relative_to(target_folder), target_path)
+
 
 class VariableDefinition(Definition):
 
@@ -129,6 +152,16 @@ class BatchDefinition(Definition):
         ]
 
 
+class DatasetDefinition(Definition):
+
+    def _initialize(self, kwargs):
+        self.automation_folder = kwargs['automation_folder']
+        self._validation_functions = [
+            validate_dataset_identifiers,
+            validate_dataset_configuration,
+        ]
+
+
 class ScriptDefinition(Definition):
 
     def _initialize(self, kwargs):
@@ -150,7 +183,7 @@ class ScriptDefinition(Definition):
             old_path = folder / script_path
             script_path = '.' + str(script_path.with_suffix('.ipynb.py'))
             new_path = folder / script_path
-            L.info('exporting %s to %s', old_path, new_path)
+            L.info('exporting %s to %s', self.path, script_path)
             try:
                 script_text = PythonExporter().from_notebook_node(
                     load_notebook(old_path, NO_CONVERT))[0]
@@ -341,18 +374,21 @@ def validate_variable_views(configuration):
 
 def validate_templates(configuration):
     template_definitions_by_mode_name = {}
-    try:
-        for mode_name in MODE_NAMES:
-            mode_configuration = get_dictionary(configuration, mode_name)
-            template_definitions = [TemplateDefinition(
-                _, mode_name=mode_name,
-            ) for _ in get_dictionaries(mode_configuration, 'templates')]
-            assert_unique_values(
-                [_.id for _ in template_definitions],
-                f'duplicate template id {{x}} in {mode_name}')
-            template_definitions_by_mode_name[mode_name] = template_definitions
-    except OSError as e:
-        raise CrossComputeConfigurationError(e)
+    automation_folder = configuration.folder
+    for mode_name in MODE_NAMES:
+        mode_configuration = get_dictionary(configuration, mode_name)
+        template_definitions = [TemplateDefinition(
+            _, mode_name=mode_name,
+        ) for _ in get_dictionaries(mode_configuration, 'templates')]
+        for template_definition in template_definitions:
+            template_path = template_definition.path
+            if not (automation_folder / template_path).exists():
+                raise CrossComputeConfigurationError(
+                    f'could not find template {template_path}')
+        assert_unique_values(
+            [_.id for _ in template_definitions],
+            f'duplicate template id {{x}} in {mode_name}')
+        template_definitions_by_mode_name[mode_name] = template_definitions
     return {
         'template_definitions_by_mode_name': template_definitions_by_mode_name,
     }
@@ -394,10 +430,20 @@ def validate_environment_variables(configuration):
     }
 
 
+def validate_datasets(configuration):
+    automation_folder = configuration.folder
+    dataset_definitions = [DatasetDefinition(
+        _, automation_folder=automation_folder,
+    ) for _ in get_dictionaries(configuration, 'datasets')]
+    return {
+        'dataset_definitions': dataset_definitions,
+    }
+
+
 def validate_scripts(configuration):
     automation_folder = configuration.folder
     script_definitions = [ScriptDefinition(
-        _, automation_folder=automation_folder
+        _, automation_folder=automation_folder,
     ) for _ in get_dictionaries(configuration, 'scripts')]
     return {
         'script_definitions': script_definitions,
@@ -462,6 +508,7 @@ def validate_template_identifiers(template_definition):
 
 
 def validate_variable_identifiers(variable_definition):
+    # TODO: Check that variable_id does not have quotes
     try:
         variable_id = variable_definition['id']
         view_name = variable_definition['view']
@@ -530,6 +577,31 @@ def validate_batch_configuration(batch_definition):
     }
 
 
+def validate_dataset_identifiers(dataset_definition):
+    path = Path(dataset_definition.get('path', '').strip())
+    return {
+        'path': path,
+    }
+
+
+def validate_dataset_configuration(dataset_definition):
+    automation_folder = dataset_definition.automation_folder
+    dataset_reference = get_dictionary(dataset_definition, 'reference')
+    if 'path' in dataset_reference:
+        source_path = Path(dataset_reference['path'].strip())
+        if not (automation_folder / source_path).exists():
+            raise CrossComputeConfigurationError(
+                f'could not find dataset reference path {source_path}')
+        target_path = dataset_definition.path
+        if target_path.exists() and not target_path.is_symlink():
+            raise CrossComputeConfigurationError(
+                'refusing to overwrite existing dataset; please delete '
+                f'{target_path} from the disk as defined')
+    return {
+        'reference': dataset_reference,
+    }
+
+
 def validate_script_identifiers(script_definition):
     folder = script_definition.get('folder', '.').strip()
 
