@@ -3,9 +3,8 @@ from argparse import ArgumentParser
 from logging import getLogger
 
 from crosscompute.exceptions import (
-    CrossComputeConfigurationError,
     CrossComputeConfigurationNotFoundError,
-    CrossComputeDataError)
+    CrossComputeError)
 from crosscompute.macros.process import LoggableProcess
 from crosscompute.routines.automation import DiskAutomation
 from crosscompute.routines.log import (
@@ -14,11 +13,16 @@ from crosscompute.routines.log import (
 from crosscompute.scripts.configure import (
     configure_argument_parser_for_configuring,
     configure_with)
+from crosscompute.scripts.print import (
+    configure_argument_parser_for_printing,
+    configure_printing_from,
+    print_with)
 from crosscompute.scripts.run import (
     configure_argument_parser_for_running,
     run_with)
 from crosscompute.scripts.serve import (
     configure_argument_parser_for_serving,
+    configure_serving_from,
     serve_with)
 
 
@@ -30,14 +34,24 @@ def do(arguments=None):
     configure_argument_parser_for_configuring(a)
     configure_argument_parser_for_serving(a)
     configure_argument_parser_for_running(a)
+    configure_argument_parser_for_printing(a)
     args = a.parse_args(arguments)
-    configure_logging_from(args)
+    try:
+        configure_logging_from(args)
+        configure_serving_from(args)
+        configure_printing_from(args)
+    except CrossComputeError as e:
+        L.error(e)
+        return
     launch_mode = get_launch_mode_from(args)
 
     if launch_mode == 'configure':
         configure_with(args)
         raise SystemExit
     automation = get_automation_from(args)
+    if launch_mode == 'print':
+        print_with(automation, args)
+        raise SystemExit
     processes = []
     if launch_mode in ['serve', 'all']:
         processes.append(LoggableProcess(
@@ -79,6 +93,8 @@ def get_launch_mode_from(args):
         launch_mode = 'run'
     elif args.is_serve_only:
         launch_mode = 'serve'
+    elif args.print_format:
+        launch_mode = 'print'
     '''
     elif args.is_debug_only:
         launch_mode = 'debug'
@@ -95,7 +111,7 @@ def get_automation_from(args):
         print()
         path = configure_with(args)
         automation = DiskAutomation.load(path)
-    except (CrossComputeConfigurationError, CrossComputeDataError) as e:
+    except CrossComputeError as e:
         L.error(e)
         raise SystemExit
     return automation
